const {Command, flags} = require('@oclif/command')
const {cli} = require('cli-ux')
const axios = require('axios')
const chalk = require('chalk')
const inquirer = require('inquirer')
const prompt = inquirer.createPromptModule()
const execa = require('execa')
const listr = require('listr')
const fs = require('fs-extra')
const path = require('path')
const mysql = require('mysql')



class PagesPayrollCliCommand extends Command {

  async run() {

    // create config directory if it does not exist
    if (!fs.existsSync(this.config.configDir)){
      fs.mkdirSync(this.config.configDir);
    }
    // read the config from configDir
    const userConfig = await fs.readJSON(path.join(this.config.configDir, 'config.json'))
    
    const {flags, args} = this.parse(PagesPayrollCliCommand)
    const headers = {
      headers: {
        'Content-Type': 'application/json',
        'Authorization': `Bearer ${userConfig.DO_ACCESS_TOKEN}`
      }
    }

    let hostname = flags.hostname || args.hostname || null
    
    const _self = this
    
    // if(!hostname || !hostname.match(/^[a-z]{3,15}$/i)){
      const response = await prompt([
        {
          type: 'input',
          name: 'hostname',
          message: chalk`Enter the {bold.yellow hostname}: `,
          default: hostname,
          validate: function(value) {
            return value.match(/^[a-z]{3,15}$/i) ? true : 'Please enter a valid hostname!';
          },
          choices: [],
        },
        {
          type: 'input',
          name: 'app_name',
          message: chalk`Enter the {bold.yellow APP_NAME}: `,
          default: `Payroll ${hostname}`,
          choices: [],
        },
        {
          type: 'input',
          name: 'app_url',
          message: chalk`Enter the {bold.yellow APP_URL}: `,
          default: `https://${hostname}.${userConfig.DOMAIN_NAME}`,
          choices: [],
        },
        {
          type: 'input',
          name: 'db_username',
          message: chalk`Enter the {bold.yellow DB_USERNAME}: `,
          default: userConfig.DB_USERNAME,
          choices: [],
        },
        {
          type: 'input',
          name: 'db_password',
          message: chalk`Enter the {bold.yellow DB_PASSWORD}: `,
          default: userConfig.DB_PASSWORD,
          choices: [],
        },
        {
          type: 'input',
          name: 'mail_username',
          message: chalk`Enter the {bold.yellow MAIL_USERNAME}: `,
          default: 'jobithmb',
          choices: [],
        },
        {
          type: 'input',
          name: 'mail_password',
          message: chalk`Enter the {bold.yellow MAIL_PASSWORD}: `,
          default: 'jobith@1993',
          choices: [],
        },
        {
          type: 'input',
          name: 'mail_from_name',
          message: chalk`Enter the {bold.yellow MAIL_FROM_NAME}: `,
          default: 'Payroll Admin',
          choices: [],
        },
        {
          type: 'input',
          name: 'mail_from_address',
          message: chalk`Enter the {bold.yellow MAIL_FROM_ADDRESS}: `,
          default: `admin@${userConfig.DOMAIN_NAME}`,
          choices: [],
        },
        {
          type: 'input',
          name: 'msg_key',
          message: chalk`Enter the {bold.yellow MSG_KEY}: `,
          choices: [],
        },
        {
          type: 'input',
          name: 'msg_sender_id',
          message: chalk`Enter the {bold.yellow MSG_SENDER_ID}: `,
          choices: [],
        }
      ])
      hostname = response.hostname
    // }
    
    // Steps for Pages Payroll Deployment
    const tasks = new listr(
      [
        // Create a new A record for new PagesPayroll Client
        {
          title: `Create a new subdomain - ${hostname}.${userConfig.DOMAIN_NAME}`,
          task: () => {
            return new listr([
              {
                title: 'Checking if subdomain already exists',
                task: (ctx, task) => {
                  axios.get(`https://api.digitalocean.com/v2/domains/${userConfig.DOMAIN_NAME}/records`, headers)
                    .then(function (response) {
                        // handle success
                        if(response.data.domain_records.some((record) => record.name === hostname)){
                          ctx.hostname_exists = true
                        }
                        else{
                          ctx.hostname_exists = false
                        }
                    })
                    .catch(function (error) {
                        // handle error
                        throw new Error(error.response.data.message)
                    })
                }
              },
              {
                title: `Create new subdomain - ${hostname}.${userConfig.DOMAIN_NAME}`,
                skip: (ctx) => {
                  return ctx.hostname_exists ? `Subdomain - ${hostname}.${userConfig.DOMAIN_NAME} already exists!` : false
                },
                task: (ctx, task) => {
                  const data = {
                    type: 'A',
                    name: hostname,
                    data: userConfig.DOMAIN_IP
                  }
                  axios.post(`https://api.digitalocean.com/v2/domains/${userConfig.DOMAIN_NAME}/records`, data, headers)
                    .then(function (response) {
                        // handle success
                        return `${hostname}.${userConfig.DOMAIN_NAME} created with ID: ${response.data.id}`
                    })
                    .catch(function (error) {
                        // handle error
                        throw new Error(error.response.data.message)
                    })
                }
              }
            ])
          }
        },
        // Create virtual host conf in apache2 sites-available 
        {
          title: `Create virtual host configurations - ${hostname}.${userConfig.DOMAIN_NAME}.conf`,
          task: () => {
            return new listr([
              {
                title: `Create a copy of the copy.${userConfig.DOMAIN_NAME}.conf`,
                task: async (ctx, task) => {
                  await execa('cp', [`${userConfig.APACHE_DIR}/copy.${userConfig.DOMAIN_NAME}.conf`, `${userConfig.APACHE_DIR}/${hostname}.${userConfig.DOMAIN_NAME}.conf`])
                  return 'success'
                }
              },
              {
                title: `Replace 'demo.${userConfig.DOMAIN_NAME}' with '${hostname}.${userConfig.DOMAIN_NAME}' in the conf file`,
                task: (ctx, task) => {
                  let filename = `${userConfig.APACHE_DIR}/${hostname}.${userConfig.DOMAIN_NAME}.conf`
                  fs.readFile(filename, 'utf8', function (err, data) {
                    if (err) {
                      return console.log(err);
                    }
                    var result = data.replace(/demo/g, hostname)

                    fs.writeFile(filename, result, 'utf8', function (err) {
                      if (err) return console.log(err);
                    })
                  })
                  return `conf file updated`
                }
              },
              {
                title: `Enable the ${hostname}.${userConfig.DOMAIN_NAME}.conf`,
                task: async (ctx, task) => {
                  process.chdir(`${userConfig.APACHE_DIR}`)
                  await execa('a2ensite', [`${hostname}.${userConfig.DOMAIN_NAME}.conf`])
                  return 'success'
                }
              },
              {
                title: `Restart apache2`,
                task: async (ctx, task) => {
                  await execa('systemctl', ['restart', 'apache2'])
                  return 'success'
                }
              },
              {
                title: `Restart apache2 service`,
                task: async (ctx, task) => {
                  await execa('service', ['apache2', 'restart'])
                  return 'success'
                }
              }
            ])
          }
        },
        // Create a new subdirectory and update the current working directory & Clone Code from BitBucket
        {
          title: `Create a new subdirectory & Clone Code from BitBucket`,
          task: () => {
            return new listr([
              {
                title: `Create a new subdirectory - ${userConfig.HOME_DIR}/${hostname}.${userConfig.DOMAIN_NAME}`,
                task: (ctx, task) => {
                  var dir = `${userConfig.HOME_DIR}/${hostname}.${userConfig.DOMAIN_NAME}`;
                  if (!fs.existsSync(dir)){
                    fs.mkdirSync(dir);
                    return 'success'
                  }
                }
              },
              {
                title: `Change the current working directory`,
                task: (ctx, task) => {
                  // change directory to the newly created subdirectory
                  process.chdir(`${userConfig.HOME_DIR}/${hostname}.${userConfig.DOMAIN_NAME}`);
                  return `Changed directory to: ${process.cwd()}`
                }
              },
              {
                title: `Clone the BitBucket Repository`,
                task: async (ctx, task) => {
                  await execa('git', ['clone', userConfig.REPOSITORY_URL, '.'])
                  return 'success'
                }
              }
            ])
          }
        },
        // Setup a new MySQL Database for the client
        {
          title: `Setup new MySQL Database - pages_${hostname}`,
          task: (ctx, task) => {
            let conn = mysql.createConnection({
              host: "localhost",
              user: userConfig.DB_USERNAME,
              password: userConfig.DB_PASSWORD
            })

            conn.connect(function(err) {
              if (err) throw err;
              conn.query(`CREATE DATABASE pages_${hostname}`, function (err, result) {
                if (err) throw err;
                return 'success'
              })
            }) 
          }
        },
        // Setup Laravel
        {
          title: `Setup Laravel`,
          task: () => {
            return new listr([
              {
                title: `Change the current working directory`,
                task: (ctx, task) => {
                  // change directory to the newly created subdirectory
                  process.chdir(`${userConfig.HOME_DIR}/${hostname}.${userConfig.DOMAIN_NAME}`);
                  return `Changed directory to: ${process.cwd()}`
                }
              },
              {
                title: `Create the .env file from .env.example`,
                task: async (ctx, task) => {
                  await execa('cp', ['.env.example', '.env'])
                  return 'success'
                }
              },
              {
                title: `Make changes in the .env file`,
                task: (ctx, task) => {
                 
                  let filename = `.env`
                  
                  fs.readFile(filename, 'utf8', function (err, data) {
                    if (err) {
                      return console.log(err);
                    }
                    let result = data.replace(/DEMO_APPNAME/g, response.app_name)
                    result = result.replace(/DEMO_URL/g, response.app_url)
                    result = result.replace(/DEMO_DATABASE/g, `pages_${hostname}`)
                    result = result.replace(/DEMO_USERNAME/g, response.db_username)
                    result = result.replace(/DEMO_PASSWORD/g, response.db_password)
                    result = result.replace(/DEMO_MAIL_USERNAME/g, response.mail_username)
                    result = result.replace(/DEMO_MAIL_PASSWORD/g, response.mail_password)
                    result = result.replace(/DEMO_FROM_NAME/g, response.mail_from_name)
                    result = result.replace(/DEMO_FROM_ADDRESS/g, response.mail_from_address)
                    result = result.replace(/DEMO_MSG_KEY/g, response.msg_key)
                    result = result.replace(/DEMO_MSG_SENDER_ID/g, response.msg_sender_id)

                    fs.writeFile(filename, result, 'utf8', function (err) {
                      if (err) return console.log(err);
                    })
                  })
                  return `env file updated`
                }
              },
              {
                title: `Composer install`,
                task: async (ctx, task) => {
                  await execa('composer', ['install'])
                  return 'success'
                }
              },
              {
                title: `Setting Permissions`,
                task: () => {
                  return new listr([
                    {
                      title: `chown storage`,
                      task: async (ctx, task) => {
                        await execa('chown', ['-R', 'root:www-data', 'storage'])
                        return 'success'
                      }
                    },
                    // {
                    //   title: `chown storage/clockwork`,
                    //   task: async (ctx, task) => {
                    //     await execa('chown', ['-R', 'root:www-data', 'storage/clockwork'])
                    //     return 'success'
                    //   }
                    // },
                    {
                      title: `chown bootstrap/cache`,
                      task: async (ctx, task) => {
                        await execa('chown', ['-R', 'root:www-data', 'bootstrap/cache'])
                        return 'success'
                      }
                    },
                    {
                      title: `chmod storage`,
                      task: async (ctx, task) => {
                        await execa('chmod', ['-R', '775', 'storage'])
                        return 'success'
                      }
                    },
                    // {
                    //   title: `chmod storage/clockwork`,
                    //   task: async (ctx, task) => {
                    //     await execa('chmod', ['-R', '775', 'storage/clockwork'])
                    //     return 'success'
                    //   }
                    // },
                    {
                      title: `chmod bootstrap/cache`,
                      task: async (ctx, task) => {
                        await execa('chmod', ['-R', '775', 'bootstrap/cache'])
                        return 'success'
                      }
                    },
                    {
                      title: `chown public/uploads`,
                      task: async (ctx, task) => {
                        await execa('chmod', ['-R', '777', 'public/uploads'])
                        return 'success'
                      }
                    },
                  ])
                }
              },
              {
                title: `Run Artisan Commands`,
                task: () => {
                  return new listr([
                    {
                      title: `php artisan migrate`,
                      task: async (ctx, task) => {
                        await execa('php', ['artisan', 'migrate'])
                        return 'success'
                      }
                    },
                    {
                      title: `php artisan db:seed`,
                      task: async (ctx, task) => {
                        await execa('php', ['artisan', 'db:seed'])
                        return 'success'
                      }
                    },
                    {
                      title: `php artisan passport:install`,
                      task: async (ctx, task) => {
                        await execa('php', ['artisan', 'passport:install'])
                        return 'success'
                      }
                    },
                    {
                      title: `php artisan route:cache`,
                      task: async (ctx, task) => {
                        await execa('php', ['artisan', 'route:cache'])
                        return 'success'
                      }
                    },
                    {
                      title: `php artisan config:cache`,
                      task: async (ctx, task) => {
                        await execa('php', ['artisan', 'config:cache'])
                        return 'success'
                      }
                    },
                  ])
                }
              }
            ])
          }
        }
      ]
    )

    // Run the Tasks
    tasks.run()
      .then(result => {
        _self.log(chalk`{bold.green Deployment Complete!}`)
        process.exit()
      })
      .catch(err => {
        this.error(err)
      })
  }

}

PagesPayrollCliCommand.description = `Automatic Deployment of Pages Payroll Web Application
...

`

PagesPayrollCliCommand.flags = {
  // add --version flag to show CLI version
  version: flags.version({char: 'v'}),
  // add --help flag to show CLI version
  help: flags.help({char: 'h'}),
  hostname: flags.string({char: 'hn'})
}

PagesPayrollCliCommand.args = [
  {
    name: 'hostname',             // name of arg to show in help and reference with args[name]
    required: false,              // make the arg required with `required: true`
    description: 'hostname for pagespayroll.app subdomain', // help description
    hidden: false,                // hide this arg from help
  }
]

module.exports = PagesPayrollCliCommand